unit process;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type


  TShowWindow = ( swHide , swMaximize , swMinimize , swRestore , swShow ,
    swShowDefault , swShowMaximized , swShowMinimized ,
    swShowMinNoActive , swShowNA , swShowNoActivate , swShowNormal );

  TProcessEvent = procedure ( Sender : TObject; ExitCode : DWord ) of object;

  TProcess = class(TComponent)
  private
    FCommand : string;
    FShowWindow : TShowWindow;
    FDirectory : string;
    FWait : Boolean;
    FOnFinished : TProcessEvent;
  protected
  public
    constructor Create( AOwner : TComponent );
    procedure Execute;
  published
    property Command : string read FCommand write FCommand;
    property Directory : string read FDirectory write FDirectory;
    property ShowWindow : TShowWindow read FShowWindow write FShowWindow;
    property Wait : Boolean read FWait write FWait;
    property OnFinished : TProcessEvent read FOnFinished write FOnFinished;
  end;

procedure Register;

implementation

const ShowWindowValues : array [ 0..11 ] of integer =
  ( sw_Hide , sw_Maximize , sw_Minimize , sw_Restore , sw_Show ,
    sw_ShowDefault , sw_ShowMaximized , sw_ShowMinimized ,
    sw_ShowMinNoActive , sw_ShowNA , sw_ShowNoActivate , sw_ShowNormal );

type

  TProcessThread = class( TThread )
  private
    FProcess : TProcess;
  protected
    procedure Execute; override;
  public
    constructor CreateThread( Process : TProcess );
  end;


procedure Register;
begin
  RegisterComponents('Samples', [TProcess]);
end;

constructor TProcess.Create( AOwner : TComponent );
begin
  inherited Create( AOwner );
  FShowWindow := swShowNormal;
  FWait := False; 
end;

procedure TProcess.Execute;
begin
  TProcessThread.CreateThread( Self );
end;

//------------------------------------------------------------------------------
//  TProcessThread

constructor TProcessThread.CreateThread( Process : TProcess );
begin
  inherited Create( True );
  FProcess := Process;
  FreeOnTerminate := True;
  Resume;
end;

procedure TProcessThread.Execute;
var
  StartupInfo : TStartupInfo;
  ProcessInfo : TProcessInformation;
  ExitCode : DWord;
  Directory : PChar;
begin
  FillChar( StartupInfo , SizeOf( StartupInfo ) , 0 );
  with StartupInfo do
    begin
      cb := SizeOf( StartupInfo );
      dwFlags := startf_UseShowWindow;
      wShowWindow := ShowWindowValues[ Ord( FProcess.ShowWindow ) ];
    end;
  ExitCode := 0;
  FProcess.Directory := Trim( FProcess.Directory );
  if Length( FProcess.Directory ) = 0 then
    Directory := nil
  else
    Directory := PChar( FProcess.Directory );

  if CreateProcess( nil , PChar( FProcess.Command ) , nil , nil , False ,
    NORMAL_PRIORITY_CLASS , nil , Directory ,
    StartupInfo , ProcessInfo ) then
    begin
      if FProcess.Wait then
        begin
          WaitForSingleObject( ProcessInfo.hProcess , Infinite );
          GetExitCodeProcess( ProcessInfo.hProcess , ExitCode );
          if Assigned( FProcess.FOnFinished ) then
            FProcess.FOnFinished( FProcess , ExitCode );
        end;
      CloseHandle( ProcessInfo.hProcess );
      CloseHandle( ProcessInfo.hThread );
    end;
end;

end.
